@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
- * Copyright (C) 2008, Nokia
+ * Copyright (C) 2006, Jamie McCracken <jamiemcc@gnome.org>
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -35,6 +35,8 @@
 #include "tracker-resources-glue.h"
 #include "tracker-resource-class.h"
 #include "tracker-resources-class-glue.h"
+#include "tracker-status.h"
+#include "tracker-status-glue.h"
 #include "tracker-statistics.h"
 #include "tracker-statistics-glue.h"
 #include "tracker-backup.h"
@@ -44,6 +46,8 @@
 static DBusGConnection *connection;
 static DBusGProxy      *gproxy;
 static GSList          *objects;
+TrackerStatus          *notifier = NULL;
+TrackerBackup          *backup = NULL;
 
 static gboolean
 dbus_register_service (DBusGProxy  *proxy,
@@ -146,16 +150,26 @@ tracker_dbus_init (void)
 		return FALSE;
 	}
 
+	dbus_g_proxy_add_signal (gproxy, "NameOwnerChanged",
+	                         G_TYPE_STRING,
+	                         G_TYPE_STRING,
+	                         G_TYPE_STRING,
+	                         G_TYPE_INVALID);
+
 	return TRUE;
 }
 
 void
 tracker_dbus_shutdown (void)
 {
-	if (objects) {
-		g_slist_foreach (objects, (GFunc) g_object_unref, NULL);
-		g_slist_free (objects);
-		objects = NULL;
+	tracker_dbus_set_available (FALSE);
+
+	if (backup) {
+		g_object_unref (backup);
+	}
+
+	if (notifier) {
+		g_object_unref (notifier);
 	}
 
 	if (gproxy) {
@@ -179,12 +193,56 @@ name_owner_changed_cb (DBusGProxy *proxy,
 	}
 }
 
+void
+tracker_dbus_set_available (gboolean available)
+{
+	if (available) {
+		if (!objects) {
+			tracker_dbus_register_objects ();
+		}
+	} else {
+		if (objects) {
+			dbus_g_proxy_disconnect_signal (gproxy,
+			                                "NameOwnerChanged",
+			                                G_CALLBACK (name_owner_changed_cb),
+			                                tracker_dbus_get_object (TRACKER_TYPE_RESOURCES));
+			g_slist_foreach (objects, (GFunc) g_object_unref, NULL);
+			g_slist_free (objects);
+			objects = NULL;
+		}
+	}
+}
+
 static void
 name_owner_changed_closure (gpointer  data,
                             GClosure *closure)
 {
 }
 
+TrackerStatus*
+tracker_dbus_register_notifier (void)
+{
+	if (!connection || !gproxy) {
+		g_critical ("D-Bus support must be initialized before registering objects!");
+		return FALSE;
+	}
+
+	/* Add org.freedesktop.Tracker */
+	notifier = tracker_status_new ();
+	if (!notifier) {
+		g_critical ("Could not create TrackerStatus object to register");
+		return FALSE;
+	}
+
+	dbus_register_object (connection,
+	                      gproxy,
+	                      G_OBJECT (notifier),
+	                      &dbus_glib_tracker_status_object_info,
+	                      TRACKER_STATUS_PATH);
+
+	return g_object_ref (notifier);
+}
+
 gboolean
 tracker_dbus_register_objects (void)
 {
@@ -219,12 +277,6 @@ tracker_dbus_register_objects (void)
 		return FALSE;
 	}
 
-	dbus_g_proxy_add_signal (gproxy, "NameOwnerChanged",
-	                         G_TYPE_STRING,
-	                         G_TYPE_STRING,
-	                         G_TYPE_STRING,
-	                         G_TYPE_INVALID);
-
 	dbus_g_proxy_connect_signal (gproxy, "NameOwnerChanged",
 	                             G_CALLBACK (name_owner_changed_cb),
 	                             object,
@@ -237,23 +289,27 @@ tracker_dbus_register_objects (void)
 	                      TRACKER_RESOURCES_PATH);
 	objects = g_slist_prepend (objects, object);
 
-	/* Add org.freedesktop.Tracker1.Backup */
-	object = tracker_backup_new ();
-	if (!object) {
-		g_critical ("Could not create TrackerBackup object to register");
-		return FALSE;
-	}
-
-	dbus_register_object (connection,
-	                      gproxy,
-	                      G_OBJECT (object),
-	                      &dbus_glib_tracker_backup_object_info,
-	                      TRACKER_BACKUP_PATH);
-	objects = g_slist_prepend (objects, object);
-
 	/* Reverse list since we added objects at the top each time */
 	objects = g_slist_reverse (objects);
 
+	if (backup == NULL) {
+		/* Add org.freedesktop.Tracker1.Backup */
+		backup = tracker_backup_new ();
+
+		if (!object) {
+			g_critical ("Could not create TrackerBackup object to register");
+			return FALSE;
+		}
+
+		dbus_register_object (connection,
+		                      gproxy,
+		                      G_OBJECT (backup),
+		                      &dbus_glib_tracker_backup_object_info,
+		                      TRACKER_BACKUP_PATH);
+		/* Backup object isn't part of the linked list, set_available wouldn't
+		 * work correctly from the dbus call otherwise */
+	}
+
 	result_set = tracker_data_query_sparql ("SELECT ?class WHERE { ?class tracker:notify true }", NULL);
 
 	if (!result_set) {
@@ -311,7 +367,7 @@ tracker_dbus_register_objects (void)
 		g_free (replaced);
 
 		/* Add a org.freedesktop.Tracker1.Resources.Class */
-		object = tracker_resource_class_new (rdf_class);
+		object = tracker_resource_class_new (rdf_class, path, connection);
 		if (!object) {
 			g_critical ("Could not create TrackerResourcesClass object to register:'%s' class",
 			            rdf_class);
@@ -351,6 +407,14 @@ tracker_dbus_get_object (GType type)
 		}
 	}
 
+	if (notifier && type == TRACKER_TYPE_STATUS) {
+		return G_OBJECT (notifier);
+	}
+
+	if (backup && type == TRACKER_TYPE_BACKUP) {
+		return G_OBJECT (backup);
+	}
+
 	return NULL;
 }
 
